pro pltgrdedge
;
;  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;  *  PROGRAM PLTGRDEDGE:  PLOTS GRID THAT IS CONSTRUCTED ONLY OF      *
;  *  TRIANGULAR ELEMENTS. GRID IS SAVED AS A "GIF" IMAGE.             *
;  *  COORDINATES OF THE ORIGIN OR THE FIRST NODE POINT IS             *
;  *  ASSUMED TO BE LOCATED IN THE LOWER LEFT-HAND-CORNER OF THE       *
;  *  GRID. GRID DIVIDED INTO THREE REGIONS:                           *
;  *                                                                   *
;  *         REGION-A: FIVE COLUMNS FROM STRESS FREE-EDGE,             *
;  *         REGION-B: INTERMEDIATE NEXT TO EDGE-REGION-A,             *
;  *         REGION-C: INTERMEDIATE NEXT TO INTERIOR-REGION-D,         *
;  *         REGION-D: INTERIOR REGION.                                *
;  *                                                                   * 
;  *  WRITTEN BY R.D.KRIZ, FALL 2000. VA TECH, BLACKSBURG, VIRGINIA    *
;  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;
 openr,5,'lfemie_mech.DAT' 
 f=string('') 
 hgtz=fltarr(3) & hgth=fltarr(3) & hgt=fltarr(2)
 hi=fltarr(438) & zi=fltarr(438)
 n1=intarr(768) & n2=intarr(768) & n3=intarr(768)
;
; Scale factor for h-coord, deg/rad conversion, small number
;
;hfactor=1.400000/12.000000
 hfactor=1.0
 rtd=180./acos(-1.)
 sn=+0.100e-07
;
; Skip 5 lines of file header
;
 for iskip=0,4 do begin
  readf,5,f
 endfor
;
;   READ FIRST LINE OF INPUT FOR GRID INFO.
;      NODES , NUMBER OF NODES
;      NELEM , NUMBER OF ELEMENTS
 readf,5,format='(2I6)',nelem,nodes
 nodes=fix(nodes)
 nelem=fix(nelem)
 nodesm1=nodes-1
 nelemm1=nelem-1
 print,nodes,nelem,nodesm1,nelemm1
; 
; Skip the next 9 lines in file before reading topology 
 for iskip=0,8 do begin
  readf,5,f
 endfor
;
;        READ NODE POINT COORDINATES
 for ind=0,nodesm1 do begin
   readf,5,format='(15x,2F12.4)',h,z
   hi(ind)=h*hfactor & zi(ind)=z
 endfor
;
;  INITIALIZE TOTAL AREA
 areat=0.0
;
;      READ ELEMENT AND ASSOCIATED NODE NO. TOPOLOGY
 for iel=0,nelemm1 do begin
   readf,5,format='(6x,3I6)',a,b,c
   n1(iel)=a & n2(iel)=b & n3(iel)=c
   aa=n1(iel)-1 & bb=n2(iel)-1 & cc=n3(iel)-1
;endfor
;
; CHECK FOR:
;
; 1. ELEMENTS WITH SAME NODE NO.S
  if((a eq b) or (a eq c) or (b eq c)) then $ 
  print,'ERROR: ELEMENT ',iel,' SHARES SAME NODE NO.S',a,b,c
;
; 2. NEGATIVE ELEMENT AREAS
  term1=-(-hi(aa)+hi(bb))*(zi(aa)+zi(bb))/2.0
  term2=-(-hi(bb)+hi(cc))*(zi(bb)+zi(cc))/2.0
  term3=-(-hi(cc)+hi(aa))*(zi(cc)+zi(aa))/2.0
  area=term1+term2+term3
  areat=areat+area
  if(area lt sn) then $
  print,'ERROR: NEGATIVE AREA FOR ELEMENT ',iel,$
        ' AREA ',area,' NODE NUMBERS ',a,b,c
;
; 3. ELEMENTS WITH 90 DEG OR EQUAL ANGLES: VECTOR DOT PRODUCTS
; mag1=sqrt((hi(bb)-hi(aa))^2+(zi(bb)-zi(aa))^2)
; mag2=sqrt((hi(cc)-hi(bb))^2+(zi(cc)-zi(bb))^2)
; mag3=sqrt((hi(aa)-hi(cc))^2+(zi(aa)-zi(cc))^2)
; dv1v2=(hi(bb)-hi(aa))*(hi(cc)-hi(bb))+(zi(bb)-zi(aa))*(zi(cc)-zi(bb))
; dv2v3=(hi(cc)-hi(bb))*(hi(aa)-hi(cc))+(zi(cc)-zi(bb))*(zi(aa)-zi(cc))
; dv3v1=(hi(aa)-hi(cc))*(hi(bb)-hi(aa))+(zi(aa)-zi(cc))*(zi(bb)-zi(aa))
; arga=dv1v2/(mag1*mag2) & argb=dv2v3/(mag2*mag3) & argc=dv3v1/(mag3*mag1)
; print,iel,arga,argb,argc
; anga=rtd*acos(arga) 
; angb=rtd*acos(argb)
; angc=rtd*acos(argc)
; tang=anga+angb+angc
; print,iel,anga,angb,angc,tang,aa,bb,cc
; diffab=abs(anga-angb) & diffbc=abs(angb-angc) & diffca=abs(angc-anga)
; if((abs(arga) lt sn) or (abs(argb) lt sn) or (abs(argc) lt sn)) then $ 
;   print,'ELEMENT ',iel,' IS A RIGHT TRIANGLE',anga,angb,angc,tang
; if((diffab lt sn) or (diffbc lt sn) or (diffca lt sn)) then $
;   print,'ELEMENT ',iel,' HAS TWO EQUAL ANGLES',anga,angb,angc,tang
;
 endfor
;
 print,' TOTAL AREA=',areat
;
 !P.BACKGROUND=255 & TEK_COLOR
;

;------------------------------------------------------------------------
;            Plot Entire Grid with "N"o Element "N"umbers ("nn")
;
;  Define mins and maxs for X and Y coordinates
 min_x=-0.03 & max_x=8.05
 min_y=-0.05 & max_y=2.05
;
; Set X and Y data Scales as PV-Wave system variables
 !X.S = [-min_x, 1.]/(max_x-min_x)
 !Y.S = [-min_y, 1.]/(max_y-min_y)
;
;  Setup Z buffer for "gif" image grid plot
;set_plot,'pm' & openzbuff,2000,300 ; can veiw smallest edge element
 set_plot,'pm' & openzbuff,850,500  ; fits on a web page nicely
;
;                       PLOT ELEMENTS
 for ie=0,nelemm1 do begin
   plots,[hi(n1(ie)-1),hi(n2(ie)-1)],[zi(n1(ie)-1),zi(n2(ie)-1)],color=0
   plots,[hi(n2(ie)-1),hi(n3(ie)-1)],[zi(n2(ie)-1),zi(n3(ie)-1)],color=0
   plots,[hi(n3(ie)-1),hi(n1(ie)-1)],[zi(n3(ie)-1),zi(n1(ie)-1)],color=0
;
 endfor
;
    print,' End of plot for entire grid with no element numbers'
;
 closezbuff,'pltgrdedge_nn.gif'
;
;------------------------------------------------------------------------
;            Plot Entire "Large" Grid with Element Numbers
;
;  Define mins and maxs for X and Y coordinates
;min_x=-0.03 & max_x=8.05
;min_y=-0.05 & max_y=2.05
;min_x=-0.03 & max_x=8.05
;min_y=-0.05 & max_y=2.05
;
; Set X and Y data Scales as PV-Wave system variables
;!X.S = [-min_x, 1.]/(max_x-min_x)
;!Y.S = [-min_y, 1.]/(max_y-min_y)
;
;  Setup Z buffer for "gif" image grid plot
;set_plot,'pm' & openzbuff,8000,2000
;
;                       PLOT ELEMENTS
;for ie=0,nelemm1 do begin
;  plots,[hi(n1(ie)-1),hi(n2(ie)-1)],[zi(n1(ie)-1),zi(n2(ie)-1)],color=0
;  plots,[hi(n2(ie)-1),hi(n3(ie)-1)],[zi(n2(ie)-1),zi(n3(ie)-1)],color=0
;  plots,[hi(n3(ie)-1),hi(n1(ie)-1)],[zi(n3(ie)-1),zi(n1(ie)-1)],color=0
;
;   Calculate centroid location for each element
;  have=(hi(n1(ie)-1)+hi(n2(ie)-1)+hi(n3(ie)-1))/3.
;  zave=(zi(n1(ie)-1)+zi(n2(ie)-1)+zi(n3(ie)-1))/3.
;
;   Calculate element number height based on the size of each element
;  hgtz(0)=abs(zi(n2(ie)-1)-zi(n1(ie)-1))
;  hgtz(1)=abs(zi(n3(ie)-1)-zi(n1(ie)-1))
;  hgtz(2)=abs(zi(n3(ie)-1)-zi(n2(ie)-1))
;  hgt(0)=max(hgtz)
;  hgth(0)=abs(hi(n2(ie)-1)-hi(n1(ie)-1))
;  hgth(1)=abs(hi(n3(ie)-1)-hi(n1(ie)-1))
;  hgth(2)=abs(hi(n3(ie)-1)-hi(n2(ie)-1))
;  hgt(1)=max(hgth)
;  hg=max(hgt)
;  txtsize=hg*15.0
;  hg=hg/12.0
;
;   Shift and center element number within each element
;  hlet=hg/2.
;  al=hlet
;  if(ie ge 0)then al=hlet*8.
;  if(ie gt 10)then al=hlet*7.
;  if(ie gt 100)then al=hlet*6.0
;  if(ie gt 1000)then al=hlet*4.2
;  have=have-al
;  zave=zave-hlet
;  s1=string(format='(i4)',ie+1)
;  xyouts,have,zave,s1,size=txtsize,color=0
;
;endfor
;
;closezbuff,'pltgrdedge_no.gif'
;
;  print,' End of plot for grid with element no.s: Element No.=',ie
;
;------------------------------------------------------------------------
;                              Plot REGION_D
;
;  Define mins and maxs for X and Y coordinates
;min_x=-0.03 & max_x=8.05
;min_y=-0.05 & max_y=2.05
 min_x=-0.19 & max_x=6.35
 min_y=-0.05 & max_y=2.05
;
; Set X and Y data Scales as PV-Wave system variables
 !X.S = [-min_x, 1.]/(max_x-min_x)
 !Y.S = [-min_y, 1.]/(max_y-min_y)
;
;  Setup Z buffer for "gif" image grid plot 
;set_plot,'pm' & openzbuff,850,500
 set_plot,'pm' & openzbuff,750,700
;
;                       PLOT ELEMENTS 
 for ie=0,71 do begin
   plots,[hi(n1(ie)-1),hi(n2(ie)-1)],[zi(n1(ie)-1),zi(n2(ie)-1)],color=0
   plots,[hi(n2(ie)-1),hi(n3(ie)-1)],[zi(n2(ie)-1),zi(n3(ie)-1)],color=0
   plots,[hi(n3(ie)-1),hi(n1(ie)-1)],[zi(n3(ie)-1),zi(n1(ie)-1)],color=0
;
;   Calculate centroid location for each element
   have=(hi(n1(ie)-1)+hi(n2(ie)-1)+hi(n3(ie)-1))/3.
   zave=(zi(n1(ie)-1)+zi(n2(ie)-1)+zi(n3(ie)-1))/3.
;
;   Calculate element number height based on the size of each element
   hgtz(0)=abs(zi(n2(ie)-1)-zi(n1(ie)-1))
   hgtz(1)=abs(zi(n3(ie)-1)-zi(n1(ie)-1))
   hgtz(2)=abs(zi(n3(ie)-1)-zi(n2(ie)-1))
   hgt(0)=max(hgtz)
   hgth(0)=abs(hi(n2(ie)-1)-hi(n1(ie)-1))
   hgth(1)=abs(hi(n3(ie)-1)-hi(n1(ie)-1))
   hgth(2)=abs(hi(n3(ie)-1)-hi(n2(ie)-1))
   hgt(1)=max(hgth)
   hg=max(hgt)
   txtsize=hg*2.0
   hg=hg/12.0
;
;   Shift and center element number within each element
   hlet=hg/2.
   al=hlet
   if(ie ge 0)then al=hlet*10.
   if(ie gt 10)then al=hlet*7.5
   if(ie gt 100)then al=hlet*6.
   have=have-al
   zave=zave-hlet*0.55
   s1=string(format='(i4)',ie+1)
   xyouts,have,zave,s1,size=txtsize,color=0
;   
 endfor
;  
   print,' End of plot for REGION_D: Element No.=',ie
;
 closezbuff,'pltgrdedge_REGION_D.gif'
;
;------------------------------------------------------------------------
;                              Plot REGION_C
;
;  Define mins and maxs for X and Y coordinates
;min_x=-0.03 & max_x=8.05
;min_y=-0.05 & max_y=2.05
 min_x=5.97 & max_x=7.30
 min_y=-0.05 & max_y=2.05
;
; Set X and Y data Scales as PV-Wave system variables
 !X.S = [-min_x, 1.]/(max_x-min_x)
 !Y.S = [-min_y, 1.]/(max_y-min_y)
;
;  Setup Z buffer for "gif" image grid plot
;set_plot,'pm' & openzbuff,850,500
 set_plot,'pm' & openzbuff,450,900
;
;                       PLOT ELEMENTS
 for ie=72,143 do begin
   plots,[hi(n1(ie)-1),hi(n2(ie)-1)],[zi(n1(ie)-1),zi(n2(ie)-1)],color=0
   plots,[hi(n2(ie)-1),hi(n3(ie)-1)],[zi(n2(ie)-1),zi(n3(ie)-1)],color=0
   plots,[hi(n3(ie)-1),hi(n1(ie)-1)],[zi(n3(ie)-1),zi(n1(ie)-1)],color=0
;
;   Calculate centroid location for each element
   have=(hi(n1(ie)-1)+hi(n2(ie)-1)+hi(n3(ie)-1))/3.
   zave=(zi(n1(ie)-1)+zi(n2(ie)-1)+zi(n3(ie)-1))/3.
;
;   Calculate element number height based on the size of each element
   hgtz(0)=abs(zi(n2(ie)-1)-zi(n1(ie)-1))
   hgtz(1)=abs(zi(n3(ie)-1)-zi(n1(ie)-1))
   hgtz(2)=abs(zi(n3(ie)-1)-zi(n2(ie)-1))
   hgt(0)=max(hgtz)
   hgth(0)=abs(hi(n2(ie)-1)-hi(n1(ie)-1))
   hgth(1)=abs(hi(n3(ie)-1)-hi(n1(ie)-1))
   hgth(2)=abs(hi(n3(ie)-1)-hi(n2(ie)-1))
   hgt(1)=max(hgth)
   hg=max(hgt)
   txtsize=hg*5.0
   hg=hg/10.0
;
;   Shift and center element number within each element
   hlet=hg/2.
   al=hlet
   if(ie ge 0)then al=hlet*8.
   if(ie gt 10)then al=hlet*5.
   if(ie gt 100)then al=hlet*5.5
   have=have-al
   zave=zave-hlet
   s1=string(format='(i4)',ie+1)
   xyouts,have,zave,s1,size=txtsize,color=0
;  
 endfor
;  
   print,' End of plot for REGION_C: Element No.=',ie
;
 closezbuff,'pltgrdedge_REGION_C.gif'
;
;------------------------------------------------------------------------
;                              Plot REGION_B
;
;  Define mins and maxs for X and Y coordinates
;min_x=-0.03 & max_x=8.05
;min_y=-0.05 & max_y=2.05
 min_x=7.17 & max_x=7.82
 min_y=-0.05 & max_y=2.05
;
; Set X and Y data Scales as PV-Wave system variables
 !X.S = [-min_x, 1.]/(max_x-min_x)
 !Y.S = [-min_y, 1.]/(max_y-min_y)
;
;  Setup Z buffer for "gif" image grid plot
;set_plot,'pm' & openzbuff,850,500
 set_plot,'pm' & openzbuff,700,1500
;
;                       PLOT ELEMENTS
 for ie=144,287 do begin
   plots,[hi(n1(ie)-1),hi(n2(ie)-1)],[zi(n1(ie)-1),zi(n2(ie)-1)],color=0
   plots,[hi(n2(ie)-1),hi(n3(ie)-1)],[zi(n2(ie)-1),zi(n3(ie)-1)],color=0
   plots,[hi(n3(ie)-1),hi(n1(ie)-1)],[zi(n3(ie)-1),zi(n1(ie)-1)],color=0
;
;   Calculate centroid location for each element
   have=(hi(n1(ie)-1)+hi(n2(ie)-1)+hi(n3(ie)-1))/3.
   zave=(zi(n1(ie)-1)+zi(n2(ie)-1)+zi(n3(ie)-1))/3.
;
;   Calculate element number height based on the size of each element
   hgtz(0)=abs(zi(n2(ie)-1)-zi(n1(ie)-1))
   hgtz(1)=abs(zi(n3(ie)-1)-zi(n1(ie)-1))
   hgtz(2)=abs(zi(n3(ie)-1)-zi(n2(ie)-1))
   hgt(0)=max(hgtz)
   hgth(0)=abs(hi(n2(ie)-1)-hi(n1(ie)-1))
   hgth(1)=abs(hi(n3(ie)-1)-hi(n1(ie)-1))
   hgth(2)=abs(hi(n3(ie)-1)-hi(n2(ie)-1))
   hgt(1)=max(hgth)
   hg=max(hgt)
   txtsize=hg*12.0
   hg=hg/12.0
;
;   Shift and center element number within each element
   hlet=hg/2.
   al=hlet
   if(ie ge 0)then al=hlet*8.
   if(ie gt 10)then al=hlet*7.
   if(ie gt 100)then al=hlet*4.0
   if(ie gt 1000)then al=hlet*4.2
   have=have-al
   zave=zave-hlet*1.3
   s1=string(format='(i4)',ie+1)
   xyouts,have,zave,s1,size=txtsize,color=0
;
 endfor
;
 closezbuff,'pltgrdedge_REGION_B.gif'
;
   print,' End of plot for REGION_B: Element No.=',ie
;
;------------------------------------------------------------------------
;                              Plot REGION_A
;
;  Define mins and maxs for X and Y coordinates
;min_x=-0.03 & max_x=8.05
;min_y=-0.05 & max_y=2.05
 min_x=7.769 & max_x=8.010
 min_y=-0.02 & max_y=2.02
;
; Set X and Y data Scales as PV-Wave system variables
 !X.S = [-min_x, 1.]/(max_x-min_x)
 !Y.S = [-min_y, 1.]/(max_y-min_y)
;
;  Setup Z buffer for "gif" image grid plot
;set_plot,'pm' & openzbuff,850,500
 set_plot,'pm' & openzbuff,700,3500
;
;                       PLOT ELEMENTS
 for ie=288,767 do begin
   plots,[hi(n1(ie)-1),hi(n2(ie)-1)],[zi(n1(ie)-1),zi(n2(ie)-1)],color=0
   plots,[hi(n2(ie)-1),hi(n3(ie)-1)],[zi(n2(ie)-1),zi(n3(ie)-1)],color=0
   plots,[hi(n3(ie)-1),hi(n1(ie)-1)],[zi(n3(ie)-1),zi(n1(ie)-1)],color=0
;
;   Calculate centroid location for each element
   have=(hi(n1(ie)-1)+hi(n2(ie)-1)+hi(n3(ie)-1))/3.
   zave=(zi(n1(ie)-1)+zi(n2(ie)-1)+zi(n3(ie)-1))/3.
;
;   Calculate element number height based on the size of each element
   hgtz(0)=abs(zi(n2(ie)-1)-zi(n1(ie)-1))
   hgtz(1)=abs(zi(n3(ie)-1)-zi(n1(ie)-1))
   hgtz(2)=abs(zi(n3(ie)-1)-zi(n2(ie)-1))
   hgt(0)=max(hgtz)
   hgth(0)=abs(hi(n2(ie)-1)-hi(n1(ie)-1))
   hgth(1)=abs(hi(n3(ie)-1)-hi(n1(ie)-1))
   hgth(2)=abs(hi(n3(ie)-1)-hi(n2(ie)-1))
   hgt(1)=max(hgth)
   hg=max(hgt)
   txtsize=hg*35.0
   hg=hg/15.0
;
;   Shift and center element number within each element
   hlet=hg/2.
   al=hlet
   if(ie ge 0)then al=hlet*8.
   if(ie gt 10)then al=hlet*7.
   if(ie gt 100)then al=hlet*4.8
   if(ie gt 1000)then al=hlet*4.2
   have=have-al
   zave=zave-hlet*2.2
   s1=string(format='(i4)',ie+1)
   xyouts,have,zave,s1,size=txtsize,color=0
;
 endfor
;
 closezbuff,'pltgrdedge_REGION_A.gif'
;
   print,' End of plot for REGION_A: Element No.=',ie
;
;-------------------------------------------------------------------------
;
 close,5
 end 
